﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.Diagnostics;
using System.Windows.Forms;
using System.IO;

namespace LumberyardApplicationBuilder
{
    public partial class Form1 : Form
    {
        String project_path;
        String project_name;
        public static String dev_path = @"C:\Amazon\Lumberyard\1.6.0.0\dev";
        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            if (fbd.ShowDialog()== System.Windows.Forms.DialogResult.OK)
            {
                project_path = fbd.SelectedPath;
                textBox1.Text = project_name = Path.GetFileName(fbd.SelectedPath);
                Application.DoEvents();
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            checkBox1.Checked = false;
            checkBox2.Checked = false;
            checkBox3.Checked = false;
            checkBox4.Checked = false;
            checkBox5.Checked = false;
            checkBox6.Checked = false;
            checkBox7.Checked = false;

            backgroundWorker1.RunWorkerAsync(project_name);
        }

        public void AppendText(String text)
        {
            Color textcolour = Color.White;
            if (text.Contains("WAF"))
            {
                textcolour = Color.GreenYellow;
            }
            if (text.Contains("WARN"))
            {
                textcolour = Color.Yellow;
            }
            if (text.Contains("ERROR"))
            {
                textcolour = Color.Red;
            }

            richTextBox1.SelectionStart = richTextBox1.TextLength;
            richTextBox1.SelectionLength = 0;

            richTextBox1.SelectionColor = textcolour;
            richTextBox1.AppendText(text);
            richTextBox1.SelectionColor = richTextBox1.ForeColor;
            richTextBox1.AppendText(Environment.NewLine);
            richTextBox1.HideSelection = false;
            richTextBox1.Invalidate();
            Application.DoEvents();
        }


        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            String game = (String)e.Argument;

            ProfileBuild();
            backgroundWorker1.ReportProgress(1);
            ReleaseBuild();
            backgroundWorker1.ReportProgress(2);
            BuildPaks(game);
            backgroundWorker1.ReportProgress(3);
            MakePaks(game);
            backgroundWorker1.ReportProgress(4);
            CompileShaders(game);
            PackShaders(game);
            backgroundWorker1.ReportProgress(5);
            CopyPAKs(game);
            backgroundWorker1.ReportProgress(6);
            CopyEXE(game);
            backgroundWorker1.ReportProgress(7);
            
        }

        private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            switch(e.ProgressPercentage)
            {
                case 1:
                    checkBox1.Checked = true;
                    break;
                case 2:
                    checkBox2.Checked = true;
                    break;
                case 3:
                    checkBox3.Checked = true;
                    break;
                case 5:
                    checkBox4.Checked = true;
                    break;
                case 6:
                    checkBox5.Checked = true;
                    break;
                case 7:
                    checkBox6.Checked = true;
                    break;
                case 4:
                    checkBox8.Checked = true;
                    break;
                
            }
        }

        private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            checkBox7.Checked = true;
        }

        private void ProfileBuild()
        {
            this.Invoke((MethodInvoker)delegate
            {
                AppendText("Starting profile build");
            });

            Process cmd = new Process();
            cmd.StartInfo.FileName = "cmd.exe";
            cmd.StartInfo.RedirectStandardInput = false;
            cmd.StartInfo.RedirectStandardOutput = true;
            cmd.StartInfo.RedirectStandardError = true;
            cmd.StartInfo.CreateNoWindow = true;
            cmd.StartInfo.UseShellExecute = false;
            cmd.StartInfo.Arguments = "/C lmbr_waf build_win_x64_profile -p all";
            cmd.StartInfo.WorkingDirectory = Form1.dev_path;
            try 
            {
                cmd.Start();
                while (!cmd.HasExited)
                {
                    string standard_output;
                    while ((standard_output = cmd.StandardOutput.ReadLine()) != null)
                    {
                        
                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output); 
                        });

                    }
                    while ((standard_output = cmd.StandardError.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output); 
                        });

                    }
                }
            }catch (Exception e)
            {
                this.Invoke((MethodInvoker)delegate
                {
                    richTextBox1.Text += e.ToString();
                });
            }
            
        }

        private void ReleaseBuild()
        {
            this.Invoke((MethodInvoker)delegate
            {
                AppendText("Starting release build");
            });
            Process cmd = new Process();
            cmd.StartInfo.FileName = "cmd.exe";
            cmd.StartInfo.RedirectStandardInput = false;
            cmd.StartInfo.RedirectStandardOutput = true;
            cmd.StartInfo.RedirectStandardError = true;
            cmd.StartInfo.CreateNoWindow = true;
            cmd.StartInfo.UseShellExecute = false;
            cmd.StartInfo.Arguments = "/C lmbr_waf build_win_x64_release -p all";
            cmd.StartInfo.WorkingDirectory = Form1.dev_path;
            try
            {
                cmd.Start();
                while (!cmd.HasExited)
                {
                    string standard_output;
                    while ((standard_output = cmd.StandardOutput.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                    while ((standard_output = cmd.StandardError.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                }
            }
            catch (Exception e)
            {
                this.Invoke((MethodInvoker)delegate
                {
                    richTextBox1.Text += e.ToString();
                });
            }

        }

        private void BuildPaks(String game)
        {
            this.Invoke((MethodInvoker)delegate
            {
                AppendText("Building PAK's");
            });
            Process cmd = new Process();
            cmd.StartInfo.FileName = "cmd.exe";
            cmd.StartInfo.RedirectStandardInput = false;
            cmd.StartInfo.RedirectStandardOutput = true;
            cmd.StartInfo.RedirectStandardError = true;
            cmd.StartInfo.CreateNoWindow = true;
            cmd.StartInfo.UseShellExecute = false;
            cmd.StartInfo.Arguments = String.Format(@"/C .\bin64\AssetProcessorBatch.exe /gamefolder={0} /platforms=pc", game);
            cmd.StartInfo.WorkingDirectory = Form1.dev_path;
            try
            {
                cmd.Start();
                while (!cmd.HasExited)
                {
                    string standard_output;
                    while ((standard_output = cmd.StandardOutput.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                    while ((standard_output = cmd.StandardError.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                }
            }
            catch (Exception e)
            {
                this.Invoke((MethodInvoker)delegate
                {
                    richTextBox1.Text += e.ToString();
                });
            }

        }

        private void MakePaks(String game)
        {
            this.Invoke((MethodInvoker)delegate
            {
                AppendText("Building PAK's");
            });
            Process cmd = new Process();
            cmd.StartInfo.FileName = "cmd.exe";
            cmd.StartInfo.RedirectStandardInput = false;
            cmd.StartInfo.RedirectStandardOutput = true;
            cmd.StartInfo.RedirectStandardError = true;
            cmd.StartInfo.CreateNoWindow = true;
            cmd.StartInfo.UseShellExecute = false;
            cmd.StartInfo.Arguments = String.Format(@"/C .\bin64\rc\rc.exe /job=bin64\rc\RCJob_Generic_MakePaks.xml /p=pc /game={0}", game.ToLower());
            cmd.StartInfo.WorkingDirectory = Form1.dev_path;
            try
            {
                cmd.Start();
                while (!cmd.HasExited)
                {
                    string standard_output;
                    while ((standard_output = cmd.StandardOutput.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                    while ((standard_output = cmd.StandardError.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                }
            }
            catch (Exception e)
            {
                this.Invoke((MethodInvoker)delegate
                {
                    richTextBox1.Text += e.ToString();
                });
            }

        }

        private void CompileShaders(String game)
        {
            this.Invoke((MethodInvoker)delegate
            {
                AppendText("Compiling shaders");
            });

           
            Process cmd = new Process();
            cmd.StartInfo.FileName = "cmd.exe";
            cmd.StartInfo.RedirectStandardInput = false;
            cmd.StartInfo.RedirectStandardOutput = true;
            cmd.StartInfo.RedirectStandardError = true;
            cmd.StartInfo.CreateNoWindow = true;
            cmd.StartInfo.UseShellExecute = false;
            cmd.StartInfo.Arguments = String.Format(@"/C .\Bin64\ShaderCacheGen.exe /BuildGlobalCache /ShadersPlatform=PC");
            cmd.StartInfo.WorkingDirectory = Form1.dev_path;
            try
            {
                cmd.Start();
                while (!cmd.HasExited)
                {
                    string standard_output;
                    while ((standard_output = cmd.StandardOutput.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                    while ((standard_output = cmd.StandardError.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                }
            }
            catch (Exception e)
            {
                this.Invoke((MethodInvoker)delegate
                {
                    richTextBox1.Text += e.ToString();
                });
            }

        }

        private void PackShaders(String game)
        {
            this.Invoke((MethodInvoker)delegate
            {
                AppendText("Packing shaders");
            });


            Process cmd = new Process();
            cmd.StartInfo.FileName = "cmd.exe";
            cmd.StartInfo.RedirectStandardInput = false;
            cmd.StartInfo.RedirectStandardOutput = true;
            cmd.StartInfo.RedirectStandardError = true;
            cmd.StartInfo.CreateNoWindow = true;
            cmd.StartInfo.UseShellExecute = false;
            cmd.StartInfo.Arguments = String.Format(@"/C .\Tools\PakShaders.bat D3D11 {0}",game);
            cmd.StartInfo.WorkingDirectory = Form1.dev_path;
            try
            {
                cmd.Start();
                while (!cmd.HasExited)
                {
                    string standard_output;
                    while ((standard_output = cmd.StandardOutput.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                    while ((standard_output = cmd.StandardError.ReadLine()) != null)
                    {

                        this.Invoke((MethodInvoker)delegate
                        {
                            AppendText(standard_output);
                        });

                    }
                }
            }
            catch (Exception e)
            {
                this.Invoke((MethodInvoker)delegate
                {
                    richTextBox1.Text += e.ToString();
                });
            }

        }

        private void CopyPAKs(String game)
        {
            // First copy the shaders
            String src = String.Format(@"C:\Amazon\Lumberyard\1.6.0.0\dev\Build\pc\{0}\D3D11\",game);
            String dest = String.Format(@"C:\Amazon\Lumberyard\1.6.0.0\dev\{0}_pc_paks\{1}\", game.ToLower(),game.ToLower());
            File.Copy(src + "shadercache.pak", dest + "shadercache.pak", true);
            File.Copy(src + "shadercachestartup.pak", dest + "shadercachestartup.pak", true);

            // Now find and copy all the pak files
            String startpath = String.Format(@"C:\Amazon\Lumberyard\1.6.0.0\dev\{0}\", game);
            CopyFiles(startpath,dest);
        }

        private void CopyFiles(String src, String dest)
        {
            DirectoryInfo dir = new DirectoryInfo(src);
            foreach (FileInfo file in dir.GetFiles("*.pak",SearchOption.TopDirectoryOnly))
            {
                String srcpath = file.DirectoryName;
                if (srcpath.Length<=src.Length)
                {
                    srcpath = "";
                }
                else
                {
                    srcpath = srcpath.Substring(src.Length) + @"\";
                }

                bool copy = true;
                String s = src + srcpath + file;
                String d = dest + srcpath + file;
                if (File.Exists(d))
                {
                    DateTime stime = File.GetLastAccessTime(s);
                    DateTime dtime = File.GetLastAccessTime(d);
                    if (dtime>stime)
                    {
                        copy = false;
                    }
                }
                if (copy)
                {
                    this.Invoke((MethodInvoker)delegate
                    {
                        AppendText(String.Format("Copying '{0}' to '{1}'", s, d));
                    });

                    String dd = Path.GetDirectoryName(d);
                    if (!Directory.Exists(dd))
                    {
                        Directory.CreateDirectory(dd);
                    }

                    File.Copy(s, d, true);
                }
                else
                {
                    this.Invoke((MethodInvoker)delegate
                    {
                        AppendText(String.Format("[WARN] Skipping '{0}'", s));
                    });
                }
            }

        }

        private void CopyEXE(string game)
        {
            String src = @"C:\Amazon\Lumberyard\1.6.0.0\dev\Bin64.Release";
            String dest = String.Format(@"C:\Amazon\Lumberyard\1.6.0.0\dev\{0}_pc_paks\Bin64.Release", game.ToLower());
            String dd = dest;
            if (!Directory.Exists(dd))
            {
                Directory.CreateDirectory(dd);
            }
            DirectoryInfo dir = new DirectoryInfo(src);
            foreach (FileInfo file in dir.GetFiles("*.*", SearchOption.AllDirectories))
            {
                String s = src + @"\" + file;
                String d = dest + @"\" + file;
                this.Invoke((MethodInvoker)delegate
                {
                    AppendText(String.Format("Copying '{0}' to '{1}'", s, d));
                });
                File.Copy(s, d, true);
            }
        }

        private void richTextBox1_TextChanged(object sender, EventArgs e)
        {
            this.Invalidate();
            Application.DoEvents();
        }
    }
}
